from unittest import TestCase

from model.data import Deck, Player, Card, Value, Color, Hand

ACE_OF_HEARTS = Card(Value.Ace, Color.Hearts)


class TestDeck(TestCase):

    def setUp(self) -> None:
        super().setUp()
        self.deck = Deck()

    def testInitDeck(self):
        self.assertEqual(52, len(self.deck), "52 cards")
        self.assertEqual(52, len(set(self.deck.cards)), "unique cards")

    def testRandomCard(self):
        cards = []
        for i in range(len(self.deck)):
            cards.append(self.deck.random_card())

        self.assertEqual(52, len(set(cards)), "draw each card once")


class TestPlayer(TestCase):

    def setUp(self) -> None:
        super().setUp()
        self.player = Player()

    def testHand(self):
        self.assertEqual(0, len(self.player.hand), "Begin no cards")
        self.assertTrue(len(self.player.name), "Has a name")

        self.player.give(ACE_OF_HEARTS)

        self.assertEqual(1, len(self.player.hand), "Gave one card")
        self.assertEqual(Value.Ace, self.player.hand[0].value, "Is Ace")
        self.assertEqual(Color.Hearts, self.player.hand[0].color, "of Hearts")

    def testDefeats(self):
        pass


class TestHand(TestCase):
    def setUp(self) -> None:
        self.hand = Hand()

    def testSimple(self):
        lowValue = Value.Two
        for value in [
            Value.Three,
            Value.Four,
            Value.Five,
            Value.Six,
            Value.Seven,
            Value.Eight,
            Value.Nine,
            Value.Ten,
            Value.Jack,
            Value.Queen,
            Value.King,
            Value.Ace,
        ]:
            highHand = Hand([Card(value, Color.Hearts)])
            lowHand = Hand([Card(lowValue, Color.Hearts)])

            self.assertGreater(highHand.value(), lowHand.value())

    def testPair(self):
        lowValue: Value = Value.Two
        for value in [
            Value.Three,
            Value.Four,
            Value.Five,
            Value.Six,
            Value.Seven,
            Value.Eight,
            Value.Nine,
            Value.Ten,
            Value.Jack,
            Value.Queen,
            Value.King,
            Value.Ace,
        ]:
            highHand: Hand = Hand([Card(value, Color.Hearts), Card(value, Color.Clubs)])
            singleHand: Hand = Hand([ACE_OF_HEARTS])
            lowHand: Hand = Hand([Card(lowValue, Color.Hearts), Card(lowValue, Color.Clubs)])

            highScore = highHand.value()
            singleScore = singleHand.value()
            lowScore = lowHand.value()

            self.assertGreater(highScore, singleScore, f"Pair[{highHand.cards}] > Ace")

            self.assertGreater(highScore, lowScore, f"Pair[{highHand.cards}] > Pair[{lowHand.cards}]]")
